
function [A,B,Uhat,Xbar] = regrCYB(U,A,B,lambda,S,nonlin,maskA,maskB)

%   [A,B,Uhat,Xbar] = regrCYB(U,A,B,lambda,S,nonlin,maskA,maskB)
%
% Function that implements adaptation in the "neocybernetic model".
%
% Inputs:
%  - U       Input data (format k x nu)
%  - A       Feedback matrix A (n x n)
%  - B       Feedforward matrix B (n x nu)
%  - lambda  Forgetting factor (scalar)
%  - S       Sparsity - how many variables used
%  - nonlin  If non-zero, nonlinear cut modeling
%  - maskA   Masking matrix for A 
%  - maskB   Masking matrix for B
% Outputs:
%  - A       Modified feedback matrix A
%  - B       Modified feedforward matrix B 
%  - Uhat    Estimate of the input
%  - Xbar    State variable sequence in balance
%
% Heikki Hytyniemi  September 1, 2005


Uhat = NaN; Xbar = NaN;
[k,nu] = size(U);
[n1,n2] = size(A);
[n,nu1] = size(B);

if nu~=nu1, disp('Matrices U and B incompatible'); return; end
if n1~=n2, disp('Matrix A non-square'); return; end
if n~=n1, disp('Matrices A and B incompatible'); return; end

if nargin<4, lambda = 1; end                % Default: No learning
if nargin<5, S = n; end                     % Default: Dense model
if nargin<6, nonlin = 0; end                % Default: Linear
if nargin<7, maskA = tril(ones(n,n)); end   % Default: Separate eigenvalues
if nargin<8, maskB = ones(n,nu); end        % Default: No effect


% Reconstruction of the "forage profiles"
invA = inv(A);
theta = B'*invA;

% Calculation of dynamic balance
Xint = U*theta;
XX = diag(sqrt(diag(A)));

if nonlin
    Xbar = Xint.*(Xint>0); 
else
    Xbar = Xint;
end
if S < n
    [sorted,order] = sort(abs(inv(XX)*Xbar'));
    Xbar = Xbar.*(order>n-S)';
end

% Adaptation of the matrices
if lambda < 1
    A = lambda*A + (1-lambda)*Xint'*Xbar/k;
    A = maskA.*A + eps*eye(n);
    B = lambda*B + (1-lambda)*Xbar'*U/k;
    B = maskB.*B;
end

% Estimates of the signals
Uhat = Xbar*theta';
